package org.bellard.eqemu;

import org.bellard.eqemu.constants.PreferenceConstants;
import org.bellard.eqemu.monitor.QEmuMonitor;
import org.bellard.eqemu.network.PortManager;
import org.bellard.eqemu.views.VMNavigationView;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.preference.PreferenceStore;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

/**
 * The activator class controls the plug-in life cycle
 */
public class Activator extends AbstractUIPlugin {

	private final static String OPTIONS = "options.properties";

	// The plug-in ID
	public static final String PLUGIN_ID = "eqemu_rcp";

	// The shared instance
	private static Activator plugin;

	private PreferenceStore options;

	private ImageManager imageManager;

	private PortManager portManager;

	private QEmuMonitor qemuMonitor;
	
	/**
	 * The constructor
	 */
	public Activator() {
		plugin = this;

	}

	public PreferenceStore getOptions() {
		return options;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
		options = new PreferenceStore();
		options.load(FileLocator.openStream(getBundle(), new Path(OPTIONS),
				false));
		imageManager = new ImageManager();
		portManager = new PortManager(getPreferenceStore().getInt(
				PreferenceConstants.PREFERENCES_MONITOR_PORT_VALUE));
		qemuMonitor = new QEmuMonitor();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
		plugin = null;
		imageManager = null;
		portManager = null;
		qemuMonitor = null;
		super.stop(context);
	}

	/**
	 * Returns the shared instance
	 * 
	 * @return the shared instance
	 */
	public static Activator getDefault() {
		return plugin;
	}

	/**
	 * Returns an image descriptor for the image file at the given plug-in
	 * relative path
	 * 
	 * @param path
	 *            the path
	 * @return the image descriptor
	 */
	public static ImageDescriptor getImageDescriptor(String path) {
		return imageDescriptorFromPlugin(PLUGIN_ID, path);
	}

	public ImageManager getImageManager() {
		return imageManager;
	}

	/**
	 * @return the portManager
	 */
	public PortManager getPortManager() {
		return portManager;
	}

	/**
	 * @return the qemuMonitor
	 */
	public QEmuMonitor getQemuMonitor() {
		return qemuMonitor;
	}

	public VMNavigationView getVMNavigationView() {
		return (VMNavigationView) Activator.getDefault()
		.getWorkbench().getActiveWorkbenchWindow().getActivePage()
		.findView(VMNavigationView.ID);
	}
	
	
}
