/**
 * 
 */
package org.bellard.eqemu.resources;

import java.io.IOException;

import org.apache.log4j.Logger;
import org.bellard.eqemu.Activator;
import org.bellard.eqemu.constants.GlobalConstants;
import org.bellard.eqemu.model.VM;
import org.bellard.eqemu.views.VMNavigationView;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRunnable;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jface.preference.PreferenceStore;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PlatformUI;

/**
 * @author Eric Bellard - eric.bellard@gmail.com
 * 
 */
public class RessourcesUtils {

	private final static Logger logger = Logger
			.getLogger(RessourcesUtils.class);

	public final static IWorkspaceRunnable createRenameVMOperation(
			final String oldname, final String newname) {

		IWorkspace workspace = ResourcesPlugin.getWorkspace();
		final IProject project = workspace.getRoot().getProject(oldname);

		IWorkspaceRunnable operation = new IWorkspaceRunnable() {
			public void run(IProgressMonitor monitor) throws CoreException {
				try {
					if (!project.isOpen()) {
						project.open(monitor);
					}
					IProjectDescription description = project.getDescription();
					description.setName(newname);
					project.setDescription(description, monitor);
					project.move(description, true, monitor);
				} catch (CoreException e) {
					String message = "pb when opening project";
					logger.debug(message, e);
					// TODO choice the exception propagation
				} catch (Exception e) {
					// TODO personalize the exception management
					String message = "pb when opening project";
					logger.debug(message, e);
					// TODO choice the exception propagation
				}

			}
		};
		return operation;
	}

	public final static void cloneProject(VM vm) {
		try {
			ResourcesPlugin.getWorkspace().run(createCloneOperation(vm),
					new NullProgressMonitor());
		} catch (CoreException e) {
			// TODO add specific exception handling
			String message = "";
			logger.debug(message, e);
			// TODO decide what to do with the exception
		}

	}

	public final static IWorkspaceRunnable createCloneOperation(final VM vm) {
		final IWorkspace workspace = ResourcesPlugin.getWorkspace();

		// find the new name
		String oldName = vm.getName();
		String newTmpName = "CloneOf" + oldName;
		if (workspace.getRoot().getProject(newTmpName).exists()) {

			int i = 2;
			while (workspace.getRoot().getProject(newTmpName + i).exists()) {
				i++;
			}
			newTmpName += i;
		}

		// copy the project
		final String newName = newTmpName;
		IWorkspaceRunnable operation = new IWorkspaceRunnable() {
			public void run(IProgressMonitor monitor) throws CoreException {
				try {
					vm.getProject().copy(
							workspace.getRoot().getProject(newName)
									.getFullPath(), true,
							new NullProgressMonitor());

					// add the project in the navigation view
					Activator.getDefault().getVMNavigationView().addVM(
							new VM(newName));
				} catch (CoreException e) {
					String message = "pb when opening project";
					logger.debug(message, e);
					// TODO choice the exception propagation
				} catch (Exception e) {
					// TODO personalize the exception management
					String message = "pb when opening project";
					logger.debug(message, e);
					// TODO choice the exception propagation
				}

			}
		};
		return operation;

	}

	public final static void renameProject(VM vm, final String newname) {
		String oldname = vm.getName();
		try {
			vm.setName(newname);
			logger.debug("rename project...");
			ResourcesPlugin.getWorkspace().run(
					createRenameVMOperation(oldname, newname),
					new NullProgressMonitor());
			vm.setProject(ResourcesPlugin.getWorkspace().getRoot().getProject(
					newname));
			IFile config = vm.getProject().getFile(
					GlobalConstants.VM_CONFIGURATION_NAME);

			PreferenceStore preferenceStore = new PreferenceStore(config
					.getLocation().toOSString());
			preferenceStore.load();
			vm.setPreferenceStore(preferenceStore);
		} catch (CoreException e) {
			String message = "pb during operation execution";
			logger.debug(message, e);
			// TODO add popup
			// TODO decide what to do with the exception
			// set back the old name
			vm.setName(oldname);
		} catch (IOException e) {
			// TODO add specific exception handling
			String message = "pb during operation execution";
			logger.debug(message, e);
			// TODO decide what to do with the exception
		}

	}

}
