/**
 * 
 */
package org.bellard.eqemu.wizards;

import java.io.BufferedInputStream;
import java.io.FileInputStream;

import org.apache.log4j.Logger;
import org.bellard.eqemu.Activator;
import org.bellard.eqemu.constants.Configuration2Constants;
import org.bellard.eqemu.model.VM;
import org.bellard.eqemu.utils.ErrorUtils;
import org.bellard.eqemu.views.VMNavigationView;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchWizard;

/**
 * @author France Telecom
 * @Copyright France Telecom S.A. 2006
 */
public class NewExampleProjectWizard extends Wizard implements INewWizard {

	public final static Logger logger = Logger
			.getLogger(NewExampleProjectWizard.class);

	public final static String ID = NewExampleProjectWizard.class.getName();

	private IStructuredSelection selection;

	/**
	 * 
	 */
	public NewExampleProjectWizard() {
		super();
		setNeedsProgressMonitor(true);
	}

	private NewProjectWizardPage newPage;

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.Wizard#addPages()
	 */
	@Override
	public void addPages() {
		newPage = new NewProjectWizardPage(selection);
		addPage(newPage);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.Wizard#performFinish()
	 */
	@Override
	public boolean performFinish() {

		String name = newPage.getNameText();

		try {
			// name check has been already done
			// create the new vm
			VM v = new VM(name);
			// add the vm to the view
			// copy freedosPath
			IPath freedosPath = new Path(Platform.getInstallLocation().getURL().getFile().toString()).append("freedos.dsk");
			IFile destination = v.getProject().getFile("freedos.dsk");
			destination.create(new BufferedInputStream(new FileInputStream(
					freedosPath.toOSString())), false,
					new NullProgressMonitor());
			v.getPreferenceStore().setValue(
					Configuration2Constants.IMAGE_ENABLE, true);
			v.getPreferenceStore().setValue(
					Configuration2Constants.IMAGE_VALUE,
					destination.getLocation().toString());
			VMNavigationView view = Activator.getDefault()
					.getVMNavigationView();

			view.addVM(v);

			return true;
		} catch (Exception e) {
			String message = "Problem during the project creation operation";
			logger.error(message, e);
		}
		// display error dialog
		ErrorUtils.displayErrorDialog("error.project.create");
		return false;
	}

	/**
	 * We will accept the selection in the workbench to see if we can initialize
	 * from it.
	 * 
	 * @see IWorkbenchWizard#init(IWorkbench, IStructuredSelection)
	 */
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		this.selection = selection;
	}
}
