/**
 * 
 */
package org.bellard.eqemu.wizards;

import org.bellard.eqemu.resources.Messages;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.jface.dialogs.IDialogPage;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

/**
 * @author France Telecom
 * @Copyright France Telecom S.A. 2006
 */
public class NewProjectWizardPage extends WizardPage {

	private Text nameText;

	private ISelection selection;

	private String oldName;

	/**
	 * Constructor for SampleNewWizardPage.
	 * 
	 * @param pageName
	 */
	public NewProjectWizardPage(ISelection selection) {
		super("wizardPage");
		setTitle("New VM Configuration Project...");
		setDescription("This wizard creates a new VM Configuration project.");
		this.selection = selection;
	}

	/**
	 * @see IDialogPage#createControl(Composite)
	 */
	public void createControl(Composite parent) {
		Composite container = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		container.setLayout(layout);
		layout.numColumns = 3;
		layout.verticalSpacing = 9;
		Label label = new Label(container, SWT.NULL);
		label.setText(Messages.getString("wizard.new.name"));

		nameText = new Text(container, SWT.BORDER | SWT.SINGLE);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		nameText.setLayoutData(gd);
		nameText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				dialogChanged();
			}
		});
		if (oldName != null) {
			nameText.setText(oldName);
		}
		nameText.setFocus();
		setPageComplete(false);
		setControl(container);
	}

	/**
	 * Ensures that both text fields are set.
	 */

	private void dialogChanged() {

		String name = nameText.getText();
		if (name == null || "".equals(name)) {
			updateStatus("The VM configuration name must be specified");
			return;
		}

		IProject project = ResourcesPlugin.getWorkspace().getRoot().getProject(
				nameText.getText());
		if (project.exists()) {
			updateStatus("The project already exists");
			return;
		}

		updateStatus(null);
	}

	private void updateStatus(String message) {
		setErrorMessage(message);
		setPageComplete(message == null);
	}

	public String getNameText() {
		return nameText.getText();
	}

	public void setOldName(String s) {
		this.oldName = s;
	}

}
